@@ -65,7 +65,7 @@
  *
  * @SBMAGIC_OFFSET: offset of SBMAGIC
  *
- * @FSSIZE: size of filessystem [not-implemented yet]
+ * @FSSIZE: size of filesystem [not-implemented yet]
  *
  * @SYSTEM_ID: ISO9660 system identifier
  *
@@ -109,8 +109,10 @@ static const struct blkid_idinfo *idinfos[] =
 	&lvm1_idinfo,
 	&snapcow_idinfo,
 	&verity_hash_idinfo,
+	&integrity_idinfo,
 	&luks_idinfo,
 	&vmfs_volume_idinfo,
+	&ubi_idinfo,
 
 	/* Filesystems */
 	&vfat_idinfo,
@@ -118,6 +120,7 @@ static const struct blkid_idinfo *idinfos[] =
 	&swap_idinfo,
 	&xfs_idinfo,
 	&xfs_log_idinfo,
+	&exfs_idinfo,
 	&ext4dev_idinfo,
 	&ext4_idinfo,
 	&ext3_idinfo,
@@ -185,8 +188,6 @@ const struct blkid_chaindrv superblocks_drv = {
  */
 int blkid_probe_enable_superblocks(blkid_probe pr, int enable)
 {
-	if (!pr)
-		return -1;
 	pr->chains[BLKID_CHAIN_SUBLKS].enabled = enable;
 	return 0;
 }
@@ -203,9 +204,6 @@ int blkid_probe_enable_superblocks(blkid_probe pr, int enable)
  */
 int blkid_probe_set_superblocks_flags(blkid_probe pr, int flags)
 {
-	if (!pr)
-		return -1;
-
 	pr->chains[BLKID_CHAIN_SUBLKS].flags = flags;
 	return 0;
 }
@@ -294,15 +292,12 @@ int blkid_probe_filter_superblocks_usage(blkid_probe pr, int flag, int usage)
  * blkid_known_fstype:
  * @fstype: filesystem name
  *
- * Returns: 1 for known filesytems, or 0 for unknown filesystem.
+ * Returns: 1 for known filesystems, or 0 for unknown filesystem.
  */
 int blkid_known_fstype(const char *fstype)
 {
 	size_t i;
 
-	if (!fstype)
-		return 0;
-
 	for (i = 0; i < ARRAY_SIZE(idinfos); i++) {
 		const struct blkid_idinfo *id = idinfos[i];
 		if (strcmp(id->name, fstype) == 0)
@@ -339,7 +334,7 @@ static int superblocks_probe(blkid_probe pr, struct blkid_chain *chn)
 	size_t i;
 	int rc = BLKID_PROBE_NONE;
 
-	if (!pr || chn->idx < -1)
+	if (chn->idx < -1)
 		return -EINVAL;
 
 	blkid_probe_chain_reset_values(pr, chn);
@@ -361,7 +356,7 @@ static int superblocks_probe(blkid_probe pr, struct blkid_chain *chn)
 	for ( ; i < ARRAY_SIZE(idinfos); i++) {
 		const struct blkid_idinfo *id;
 		const struct blkid_idmag *mag = NULL;
-		blkid_loff_t off = 0;
+		uint64_t off = 0;
 
 		chn->idx = i;
 		id = idinfos[i];
@@ -372,7 +367,7 @@ static int superblocks_probe(blkid_probe pr, struct blkid_chain *chn)
 			continue;
 		}
 
-		if (id->minsz && id->minsz > pr->size) {
+		if (id->minsz && (unsigned)id->minsz > pr->size) {
 			rc = BLKID_PROBE_NONE;
 			continue;	/* the device is too small */
 		}
@@ -410,7 +405,7 @@ static int superblocks_probe(blkid_probe pr, struct blkid_chain *chn)
 			}
 		}
 
-		/* all cheks passed */
+		/* all checks passed */
 		if (chn->flags & BLKID_SUBLKS_TYPE)
 			rc = blkid_probe_set_value(pr, "TYPE",
 				(unsigned char *) id->name,
@@ -440,7 +435,7 @@ static int superblocks_probe(blkid_probe pr, struct blkid_chain *chn)
 
 /*
  * This is the same function as blkid_do_probe(), but returns only one result
- * (cannot be used in while()) and checks for ambivalen results (more
+ * (cannot be used in while()) and checks for ambivalent results (more
  * filesystems on the device) -- in such case returns -2.
  *
  * The function does not check for filesystems when a RAID or crypto signature
@@ -509,7 +504,7 @@ static int superblocks_safeprobe(blkid_probe pr, struct blkid_chain *chn)
 
 	/*
 	 * The RAID device could be partitioned. The problem are RAID1 devices
-	 * where the partition table is visible from underlaying devices. We
+	 * where the partition table is visible from underlying devices. We
 	 * have to ignore such partition tables.
 	 */
 	if (chn->idx >= 0 && idinfos[chn->idx]->usage & BLKID_USAGE_RAID)
@@ -713,7 +708,7 @@ int blkid_probe_sprintf_uuid(blkid_probe pr, unsigned char *uuid,
 	return rc;
 }
 
-/* function to set UUIDs that are in suberblocks stored as strings */
+/* function to set UUIDs that are in superblocks stored as strings */
 int blkid_probe_strncpy_uuid(blkid_probe pr, unsigned char *str, size_t len)
 {
 	struct blkid_chain *chn = blkid_probe_get_chain(pr);
@@ -773,7 +768,7 @@ int blkid_probe_set_uuid_as(blkid_probe pr, unsigned char *uuid, const char *nam
 	if (!v)
 		return -ENOMEM;
 
-	v->len = 37;
+	v->len = UUID_STR_LEN;
 	v->data = calloc(1, v->len);
 	if (!v->data)
 		rc = -ENOMEM;
