@@ -238,12 +238,12 @@ static void bind_ns_files_from_child(pid_t *child, int fds[2])
 	}
 }
 
-static void usage(int status)
+static void __attribute__((__noreturn__)) usage(void)
 {
-	FILE *out = status == EXIT_SUCCESS ? stdout : stderr;
+	FILE *out = stdout;
 
 	fputs(USAGE_HEADER, out);
-	fprintf(out, _(" %s [options] <program> [<argument>...]\n"),
+	fprintf(out, _(" %s [options] [<program> [<argument>...]]\n"),
 		program_invocation_short_name);
 
 	fputs(USAGE_SEPARATOR, out);
@@ -265,11 +265,10 @@ static void usage(int status)
 	fputs(_(" -s, --setgroups allow|deny  control the setgroups syscall in user namespaces\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("unshare(1)"));
+	printf(USAGE_HELP_OPTIONS(27));
+	printf(USAGE_MAN_TAIL("unshare(1)"));
 
-	exit(status);
+	exit(EXIT_SUCCESS);
 }
 
 int main(int argc, char *argv[])
@@ -280,23 +279,23 @@ int main(int argc, char *argv[])
 		OPT_SETGROUPS
 	};
 	static const struct option longopts[] = {
-		{ "help", no_argument, 0, 'h' },
-		{ "version", no_argument, 0, 'V'},
+		{ "help",          no_argument,       NULL, 'h'             },
+		{ "version",       no_argument,       NULL, 'V'             },
 
-		{ "mount",  optional_argument, 0, 'm' },
-		{ "uts",    optional_argument, 0, 'u' },
-		{ "ipc",    optional_argument, 0, 'i' },
-		{ "net",    optional_argument, 0, 'n' },
-		{ "pid",    optional_argument, 0, 'p' },
-		{ "user",   optional_argument, 0, 'U' },
-		{ "cgroup", optional_argument, 0, 'C' },
+		{ "mount",         optional_argument, NULL, 'm'             },
+		{ "uts",           optional_argument, NULL, 'u'             },
+		{ "ipc",           optional_argument, NULL, 'i'             },
+		{ "net",           optional_argument, NULL, 'n'             },
+		{ "pid",           optional_argument, NULL, 'p'             },
+		{ "user",          optional_argument, NULL, 'U'             },
+		{ "cgroup",        optional_argument, NULL, 'C'             },
 
-		{ "fork", no_argument, 0, 'f' },
-		{ "mount-proc", optional_argument, 0, OPT_MOUNTPROC },
-		{ "map-root-user", no_argument, 0, 'r' },
-		{ "propagation", required_argument, 0, OPT_PROPAGATION },
-		{ "setgroups", required_argument, 0, OPT_SETGROUPS },
-		{ NULL, 0, 0, 0 }
+		{ "fork",          no_argument,       NULL, 'f'             },
+		{ "mount-proc",    optional_argument, NULL, OPT_MOUNTPROC   },
+		{ "map-root-user", no_argument,       NULL, 'r'             },
+		{ "propagation",   required_argument, NULL, OPT_PROPAGATION },
+		{ "setgroups",     required_argument, NULL, OPT_SETGROUPS   },
+		{ NULL, 0, NULL, 0 }
 	};
 
 	int setgrpcmd = SETGROUPS_NONE;
@@ -308,7 +307,7 @@ int main(int argc, char *argv[])
 	int status;
 	unsigned long propagation = UNSHARE_PROPAGATION_DEFAULT;
 	uid_t real_euid = geteuid();
-	gid_t real_egid = getegid();;
+	gid_t real_egid = getegid();
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -321,7 +320,7 @@ int main(int argc, char *argv[])
 			forkit = 1;
 			break;
 		case 'h':
-			usage(EXIT_SUCCESS);
+			usage();
 		case 'V':
 			printf(UTIL_LINUX_VERSION);
 			return EXIT_SUCCESS;
@@ -375,7 +374,7 @@ int main(int argc, char *argv[])
 			propagation = parse_propagation(optarg);
 			break;
 		default:
-			usage(EXIT_FAILURE);
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 
