@@ -70,10 +70,11 @@
 #include "ttyutils.h"
 #include "all-io.h"
 #include "monotonic.h"
+#include "timeutils.h"
 
 #include "debug.h"
 
-UL_DEBUG_DEFINE_MASK(script);
+static UL_DEBUG_DEFINE_MASK(script);
 UL_DEBUG_DEFINE_MASKNAMES(script) = UL_DEBUG_EMPTY_MASKNAMES;
 
 #define SCRIPT_DEBUG_INIT	(1 << 1)
@@ -112,7 +113,7 @@ struct script_control {
 	struct termios attrs;	/* slave terminal runtime attributes */
 	struct winsize win;	/* terminal window size */
 #if !HAVE_LIBUTIL || !HAVE_PTY_H
-	char line *;		/* terminal line */
+	char *line;		/* terminal line */
 #endif
 	unsigned int
 	 append:1,		/* append output */
@@ -122,7 +123,6 @@ struct script_control {
 	 timing:1,		/* include timing file */
 	 force:1,		/* write output to links */
 	 isterm:1,		/* is child process running as terminal */
-	 data_on_way:1,		/* sent data to master */
 	 die:1;			/* terminate program */
 
 	sigset_t sigset;	/* catch SIGCHLD and SIGWINCH with signalfd() */
@@ -154,8 +154,9 @@ static inline time_t script_time(time_t *t)
 # define script_time(x) time(x)
 #endif
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] [file]\n"), program_invocation_short_name);
 
@@ -163,18 +164,18 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_("Make a typescript of a terminal session.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -a, --append            append the output\n"
-		" -c, --command <command> run command rather than interactive shell\n"
-		" -e, --return            return exit code of the child process\n"
-		" -f, --flush             run flush after each write\n"
-		"     --force             use output file even when it is a link\n"
-		" -q, --quiet             be quiet\n"
-		" -t, --timing[=<file>]   output timing data to stderr (or to FILE)\n"
-		" -V, --version           output version information and exit\n"
-		" -h, --help              display this help and exit\n\n"), out);
+	fputs(_(" -a, --append                  append the output\n"
+		" -c, --command <command>       run command rather than interactive shell\n"
+		" -e, --return                  return exit code of the child process\n"
+		" -f, --flush                   run flush after each write\n"
+		"     --force                   use output file even when it is a link\n"
+		" -q, --quiet                   be quiet\n"
+		" -t[<file>], --timing[=<file>] output timing data to stderr or to FILE\n"
+		), out);
+	printf(USAGE_HELP_OPTIONS(31));
 
-	fprintf(out, USAGE_MAN_TAIL("script(1)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf(USAGE_MAN_TAIL("script(1)"));
+	exit(EXIT_SUCCESS);
 }
 
 static void die_if_link(const struct script_control *ctl)
@@ -190,13 +191,37 @@ static void die_if_link(const struct script_control *ctl)
 		       "Program not started."), ctl->fname);
 }
 
+static void restore_tty(struct script_control *ctl, int mode)
+{
+	struct termios rtt;
+
+	if (!ctl->isterm)
+		return;
+
+	rtt = ctl->attrs;
+	tcsetattr(STDIN_FILENO, mode, &rtt);
+}
+
+static void enable_rawmode_tty(struct script_control *ctl)
+{
+	struct termios rtt;
+
+	if (!ctl->isterm)
+		return;
+
+	rtt = ctl->attrs;
+	cfmakeraw(&rtt);
+	rtt.c_lflag &= ~ECHO;
+	tcsetattr(STDIN_FILENO, TCSANOW, &rtt);
+}
+
 static void __attribute__((__noreturn__)) done(struct script_control *ctl)
 {
 	DBG(MISC, ul_debug("done!"));
 
-	if (ctl->isterm)
-		tcsetattr(STDIN_FILENO, TCSADRAIN, &ctl->attrs);
-	if (!ctl->quiet)
+	restore_tty(ctl, TCSADRAIN);
+
+	if (!ctl->quiet && ctl->typescriptfp)
 		printf(_("Script done, file is %s\n"), ctl->fname);
 #ifdef HAVE_LIBUTEMPTER
 	if (ctl->master >= 0)
@@ -204,9 +229,10 @@ static void __attribute__((__noreturn__)) done(struct script_control *ctl)
 #endif
 	kill(ctl->child, SIGTERM);	/* make sure we don't create orphans */
 
-	if (ctl->timingfp)
-		fclose(ctl->timingfp);
-	fclose(ctl->typescriptfp);
+	if (ctl->timingfp && close_stream(ctl->timingfp) != 0)
+		err(EXIT_FAILURE, "write failed: %s", ctl->tname);
+	if (ctl->typescriptfp && close_stream(ctl->typescriptfp) != 0)
+		err(EXIT_FAILURE, "write failed: %s", ctl->fname);
 
 	if (ctl->rc_wanted) {
 		if (WIFSIGNALED(ctl->childstatus))
@@ -276,15 +302,10 @@ static void write_output(struct script_control *ctl, char *obuf,
 	DBG(IO, ul_debug("  writing output *done*"));
 }
 
-static void wait_for_empty_fd(int fd)
+static int write_to_shell(struct script_control *ctl,
+			  char *buf, size_t bufsz)
 {
-	struct pollfd fds[] = {
-	           { .fd = fd, .events = POLLIN }
-	};
-
-	while (poll(fds, 1, 10) == 1) {
-		DBG(IO, ul_debug("  data to read"));
-	}
+	return write_all(ctl->master, buf, bufsz);
 }
 
 /*
@@ -295,26 +316,32 @@ static void wait_for_empty_fd(int fd)
  * problem we wait until slave is empty. For example:
  *
  *   echo "date" | script
+ *
+ * Unfortunately, the child (usually shell) can ignore stdin at all, so we
+ * don't wait forever to avoid dead locks...
+ *
+ * Note that script is primarily designed for interactive sessions as it
+ * maintains master+slave tty stuff within the session. Use pipe to write to
+ * script(1) and assume non-interactive (tee-like) behavior is NOT well
+ * supported.
  */
-static int write_to_shell(struct script_control *ctl, char *buf, size_t bufsz)
-{
-	int rc;
-
-	if (ctl->data_on_way) {
-		wait_for_empty_fd(ctl->slave);
-		ctl->data_on_way = 0;
-	}
-	rc = write_all(ctl->master, buf, bufsz);
-	if (rc == 0)
-		ctl->data_on_way = 1;
-	return rc;
-
-}
-
 static void write_eof_to_shell(struct script_control *ctl)
 {
+	unsigned int tries = 0;
+	struct pollfd fds[] = {
+	           { .fd = ctl->slave, .events = POLLIN }
+	};
 	char c = DEF_EOF;
 
+	DBG(IO, ul_debug(" waiting for empty slave"));
+	while (poll(fds, 1, 10) == 1 && tries < 8) {
+		DBG(IO, ul_debug("   slave is not empty"));
+		xusleep(250000);
+		tries++;
+	}
+	if (tries < 8)
+		DBG(IO, ul_debug("   slave is empty now"));
+
 	DBG(IO, ul_debug(" sending EOF to master"));
 	write_to_shell(ctl, &c, sizeof(char));
 }
@@ -375,10 +402,19 @@ static void handle_signal(struct script_control *ctl, int fd)
 
 	switch (info.ssi_signo) {
 	case SIGCHLD:
-		wait_for_child(ctl, 0);
-		ctl->poll_timeout = 10;
+		DBG(SIGNAL, ul_debug(" get signal SIGCHLD"));
+		if (info.ssi_code == CLD_EXITED) {
+			wait_for_child(ctl, 0);
+			ctl->poll_timeout = 10;
+		} else if (info.ssi_status == SIGSTOP && ctl->child) {
+			DBG(SIGNAL, ul_debug(" child stop by SIGSTOP -- stop parent too"));
+			kill(getpid(), SIGSTOP);
+			DBG(SIGNAL, ul_debug(" resume"));
+			kill(ctl->child, SIGCONT);
+		}
 		return;
 	case SIGWINCH:
+		DBG(SIGNAL, ul_debug(" get signal SIGWINCH"));
 		if (ctl->isterm) {
 			ioctl(STDIN_FILENO, TIOCGWINSZ, (char *)&ctl->win);
 			ioctl(ctl->slave, TIOCSWINSZ, (char *)&ctl->win);
@@ -389,6 +425,7 @@ static void handle_signal(struct script_control *ctl, int fd)
 	case SIGINT:
 		/* fallthrough */
 	case SIGQUIT:
+		DBG(SIGNAL, ul_debug(" get signal SIG{TERM,INT,QUIT}"));
 		fprintf(stderr, _("\nSession terminated.\n"));
 		/* Child termination is going to generate SIGCHILD (see above) */
 		kill(ctl->child, SIGTERM);
@@ -400,13 +437,13 @@ static void handle_signal(struct script_control *ctl, int fd)
 
 static void do_io(struct script_control *ctl)
 {
-	int ret, ignore_stdin = 0, eof = 0;
+	int ret, eof = 0;
 	time_t tvec = script_time((time_t *)NULL);
 	char buf[128];
 	enum {
 		POLLFD_SIGNAL = 0,
 		POLLFD_MASTER,
-		POLLFD_STDIN	/* optional; keep it last, see ignore_stdin */
+		POLLFD_STDIN
 
 	};
 	struct pollfd pfd[] = {
@@ -416,21 +453,30 @@ static void do_io(struct script_control *ctl)
 	};
 
 
-	if ((ctl->typescriptfp = fopen(ctl->fname, ctl->append ? "a" : "w")) == NULL) {
+	if ((ctl->typescriptfp =
+	     fopen(ctl->fname, ctl->append ? "a" UL_CLOEXECSTR : "w" UL_CLOEXECSTR)) == NULL) {
+
+		restore_tty(ctl, TCSANOW);
 		warn(_("cannot open %s"), ctl->fname);
 		fail(ctl);
 	}
 	if (ctl->timing) {
-		if (!ctl->tname) {
-			if (!(ctl->timingfp = fopen("/dev/stderr", "w")))
-				err(EXIT_FAILURE, _("cannot open %s"), "/dev/stderr");
-		} else if (!(ctl->timingfp = fopen(ctl->tname, "w")))
-			err(EXIT_FAILURE, _("cannot open %s"), ctl->tname);
+		const char *tname = ctl->tname ? ctl->tname : "/dev/stderr";
+
+		if (!(ctl->timingfp = fopen(tname, "w" UL_CLOEXECSTR))) {
+			restore_tty(ctl, TCSANOW);
+			warn(_("cannot open %s"), tname);
+			fail(ctl);
+		}
 	}
 
 
-	strftime(buf, sizeof buf, "%c\n", localtime(&tvec));
-	fprintf(ctl->typescriptfp, _("Script started on %s"), buf);
+	if (ctl->typescriptfp) {
+		strtime_iso(&tvec, ISO_8601_DATE | ISO_8601_TIME |
+				ISO_8601_TIMEZONE | ISO_8601_SPACE,
+				buf, sizeof(buf));
+		fprintf(ctl->typescriptfp, _("Script started on %s\n"), buf);
+	}
 	gettime_monotonic(&ctl->oldtime);
 
 	while (!ctl->die) {
@@ -440,7 +486,7 @@ static void do_io(struct script_control *ctl)
 		DBG(POLL, ul_debug("calling poll()"));
 
 		/* wait for input or signal */
-		ret = poll(pfd, ARRAY_SIZE(pfd) - ignore_stdin, ctl->poll_timeout);
+		ret = poll(pfd, ARRAY_SIZE(pfd), ctl->poll_timeout);
 		errsv = errno;
 		DBG(POLL, ul_debug("poll() rc=%d", ret));
 
@@ -456,7 +502,7 @@ static void do_io(struct script_control *ctl)
 			break;
 		}
 
-		for (i = 0; i < ARRAY_SIZE(pfd) - ignore_stdin; i++) {
+		for (i = 0; i < ARRAY_SIZE(pfd); i++) {
 			if (pfd[i].revents == 0)
 				continue;
 
@@ -480,11 +526,9 @@ static void do_io(struct script_control *ctl)
 				if ((pfd[i].revents & POLLHUP) || eof) {
 					DBG(POLL, ul_debug(" ignore FD"));
 					pfd[i].fd = -1;
-					/* according to man poll() set FD to -1 can't be used to ignore
-					 * STDIN, so let's remove the FD from pool at all */
 					if (i == POLLFD_STDIN) {
-						ignore_stdin = 1;
 						write_eof_to_shell(ctl);
+						DBG(POLL, ul_debug("  ignore STDIN"));
 					}
 				}
 				continue;
@@ -499,10 +543,13 @@ static void do_io(struct script_control *ctl)
 
 	if (!ctl->die)
 		wait_for_child(ctl, 1);
-	if (!ctl->quiet && ctl->typescriptfp) {
+
+	if (ctl->typescriptfp) {
 		tvec = script_time((time_t *)NULL);
-		strftime(buf, sizeof buf, "%c\n", localtime(&tvec));
-		fprintf(ctl->typescriptfp, _("\nScript done on %s"), buf);
+		strtime_iso(&tvec, ISO_8601_DATE | ISO_8601_TIME |
+				ISO_8601_TIMEZONE | ISO_8601_SPACE,
+				buf, sizeof(buf));
+		fprintf(ctl->typescriptfp, _("\nScript done on %s\n"), buf);
 	}
 	done(ctl);
 }
@@ -511,7 +558,7 @@ static void getslave(struct script_control *ctl)
 {
 #ifndef HAVE_LIBUTIL
 	ctl->line[strlen("/dev/")] = 't';
-	ctl->slave = open(ctl->line, O_RDWR);
+	ctl->slave = open(ctl->line, O_RDWR | O_CLOEXEC);
 	if (ctl->slave < 0) {
 		warn(_("cannot open %s"), ctl->line);
 		fail(ctl);
@@ -576,18 +623,6 @@ static void __attribute__((__noreturn__)) do_shell(struct script_control *ctl)
 	fail(ctl);
 }
 
-static void fixtty(struct script_control *ctl)
-{
-	struct termios rtt;
-
-	if (!ctl->isterm)
-		return;
-
-	rtt = ctl->attrs;
-	cfmakeraw(&rtt);
-	rtt.c_lflag &= ~ECHO;
-	tcsetattr(STDIN_FILENO, TCSANOW, &rtt);
-}
 
 static void getmaster(struct script_control *ctl)
 {
@@ -621,7 +656,7 @@ static void getmaster(struct script_control *ctl)
 			break;
 		for (cp = "0123456789abcdef"; *cp; cp++) {
 			*pty = *cp;
-			ctl->master = open(ctl->line, O_RDWR);
+			ctl->master = open(ctl->line, O_RDWR | O_CLOEXEC);
 			if (ctl->master >= 0) {
 				char *tp = &ctl->line[strlen("/dev/")];
 				int ok;
@@ -722,10 +757,10 @@ int main(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 			break;
 		case 'h':
-			usage(stdout);
+			usage();
 			break;
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 		}
 	argc -= optind;
 	argv += optind;
@@ -744,7 +779,7 @@ int main(int argc, char **argv)
 	getmaster(&ctl);
 	if (!ctl.quiet)
 		printf(_("Script started, file is %s\n"), ctl.fname);
-	fixtty(&ctl);
+	enable_rawmode_tty(&ctl);
 
 #ifdef HAVE_LIBUTEMPTER
 	utempter_add_record(ctl.master, NULL);
@@ -761,7 +796,7 @@ int main(int argc, char **argv)
 	 * handled according to their default dispositions */
 	sigprocmask(SIG_BLOCK, &ctl.sigset, &ctl.sigorg);
 
-	if ((ctl.sigfd = signalfd(-1, &ctl.sigset, 0)) < 0)
+	if ((ctl.sigfd = signalfd(-1, &ctl.sigset, SFD_CLOEXEC)) < 0)
 		err(EXIT_FAILURE, _("cannot set signal handler"));
 
 	DBG(SIGNAL, ul_debug("signal fd=%d", ctl.sigfd));
