@@ -92,7 +92,8 @@ struct column_control {
 	size_t	maxlength;	/* longest input record (line) */
 
 	unsigned int greedy :1,
-		     json :1;
+		     json :1,
+		     header_repeat :1;
 };
 
 static size_t width(const wchar_t *str)
@@ -207,12 +208,16 @@ static void init_table(struct column_control *ctl)
 	if (ctl->json) {
 		scols_table_enable_json(ctl->tab, 1);
 		scols_table_set_name(ctl->tab, ctl->tab_name ? : "table");
-	}
+	} else
+		scols_table_enable_noencoding(ctl->tab, 1);
+
 	if (ctl->tab_colnames) {
 		char **name;
 
 		STRV_FOREACH(name, ctl->tab_colnames)
 			scols_table_new_column(ctl->tab, *name, 0, 0);
+		if (ctl->header_repeat)
+			scols_table_enable_header_repeat(ctl->tab, 1);
 	} else
 		scols_table_enable_noheadings(ctl->tab, 1);
 }
@@ -490,7 +495,7 @@ static void columnate_fillrows(struct column_control *ctl)
 	ctl->maxlength = (ctl->maxlength + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1);
 	numcols = ctl->termwidth / ctl->maxlength;
 	endcol = ctl->maxlength;
-	for (chcnt = col = 0, lp = ctl->ents;; ++lp) {
+	for (chcnt = col = 0, lp = ctl->ents; /* nothing */; ++lp) {
 		fputws(*lp, stdout);
 		chcnt += width(*lp);
 		if (!--ctl->nents)
@@ -551,9 +556,9 @@ static void simple_print(struct column_control *ctl)
 	}
 }
 
-static void __attribute__((__noreturn__)) usage(int rc)
+static void __attribute__((__noreturn__)) usage(void)
 {
-	FILE *out = rc == EXIT_FAILURE ? stderr : stdout;
+	FILE *out = stdout;
 
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] [<file>...]\n"), program_invocation_short_name);
@@ -567,6 +572,7 @@ static void __attribute__((__noreturn__)) usage(int rc)
 	fputs(_(" -O, --table-order <columns>      specify order of output columns\n"), out);
 	fputs(_(" -N, --table-columns <names>      comma separated columns names\n"), out);
 	fputs(_(" -E, --table-noextreme <columns>  don't count long text from the columns to column width\n"), out);
+	fputs(_(" -e, --table-header-repeat        repeat header for each page\n"), out);
 	fputs(_(" -H, --table-hide <columns>       don't print the columns\n"), out);
 	fputs(_(" -R, --table-right <columns>      right align text in these columns\n"), out);
 	fputs(_(" -T, --table-truncate <columns>   truncate text in the columns when necessary\n"), out);
@@ -585,11 +591,10 @@ static void __attribute__((__noreturn__)) usage(int rc)
 	fputs(_(" -x, --fillrows                   fill rows before columns\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("column(1)"));
+	printf(USAGE_HELP_OPTIONS(34));
+	printf(USAGE_MAN_TAIL("column(1)"));
 
-	exit(rc);
+	exit(EXIT_SUCCESS);
 }
 
 int main(int argc, char **argv)
@@ -621,6 +626,7 @@ int main(int argc, char **argv)
 		{ "table-right",         required_argument, NULL, 'R' },
 		{ "table-truncate",      required_argument, NULL, 'T' },
 		{ "table-wrap",          required_argument, NULL, 'W' },
+		{ "table-header-repeat", no_argument,       NULL, 'e' },
 		{ "tree",                required_argument, NULL, 'r' },
 		{ "tree-id",             required_argument, NULL, 'i' },
 		{ "tree-parent",         required_argument, NULL, 'p' },
@@ -642,7 +648,7 @@ int main(int argc, char **argv)
 	ctl.output_separator = "  ";
 	ctl.input_separator = mbs_to_wcs("\t ");
 
-	while ((c = getopt_long(argc, argv, "c:E:H:hi:JN:n:O:o:p:R:r:s:T:tVW:x", longopts, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "c:E:eH:hi:JN:n:O:o:p:R:r:s:T:tVW:x", longopts, NULL)) != -1) {
 
 		err_exclusive_options(c, longopts, excl, excl_st);
 
@@ -653,11 +659,14 @@ int main(int argc, char **argv)
 		case 'E':
 			ctl.tab_colnoextrem = optarg;
 			break;
+		case 'e':
+			ctl.header_repeat = 1;
+			break;
 		case 'H':
 			ctl.tab_colhide = optarg;
 			break;
 		case 'h':
-			usage(EXIT_SUCCESS);
+			usage();
 			break;
 		case 'i':
 			ctl.tree_id = optarg;
